#include "controlworker.h"
#include "frontend_nodes.h"
#include "frontend_param.h"
#include "graph_blocks.h"

ControlWorker::ControlWorker(QObject *parent) :
    QObject(parent), conn(NULL)
{

}

ControlWorker::~ControlWorker()
{
    disconnectBackend();
}

void ControlWorker::connectBackend()
{
    conn = ucart_backendConnect();
}

void ControlWorker::disconnectBackend()
{
    if (conn) {
         ucart_backendDisconnect(conn);
         conn = NULL;
    }
}

void ControlWorker::getNodes()
{
    if (conn) {
        frontend_node_data * nd = NULL;
        size_t num_nodes = 0;
        frontend_getnodes(conn, &nd, &num_nodes);
        QStringList nodes;
        QStringList const_block_nodes;
        for (size_t i = 0; i < num_nodes; i++) {
            nodes.append(QString(nd[i].name));
            if (nd[i].type == BLOCK_CONSTANT) {
                const_block_nodes.append(nd[i].name);
            }
        }
        frontend_free_node_data(nd, num_nodes);
        emit(gotNodes(nodes));
        emit(gotConstantBlocks(const_block_nodes));
    }
}

void ControlWorker::getParams(QString node)
{
    if (conn) {
        frontend_node_data * nd = NULL;
        size_t num_nodes = 0;
        frontend_getnodes(conn, &nd, &num_nodes);

        for (size_t i = 0; i < num_nodes; i++) {
            if (QString(nd[i].name) == node) {
                QStringList params;

                /* Get type definition */
                const struct graph_node_type * type = blockDefs[nd[i].type];
                /* Iterate through param names to append, and then emit signal */
                for (ssize_t j = 0; j < type->n_params; j++) {
                    params.append(QString(type->param_names[j]));
                }
                emit(gotParams(params));
            }
        }
        frontend_free_node_data(nd, num_nodes);
    }
}



void ControlWorker::getParamValue(QString node, QString param)
{
    if (conn) {
        frontend_node_data * nd = NULL;
        size_t num_nodes = 0;
        frontend_getnodes(conn, &nd, &num_nodes);

        for (size_t i = 0; i < num_nodes; i++) {
            if (QString(nd[i].name) == node) {
                frontend_param_data pd;
                pd.block = nd[i].block;

                /* Get the type definition, then iterate through to find the param */
                const struct graph_node_type * type = blockDefs[nd[i].type];
                for (ssize_t j = 0; j < type->n_params; j++) {
                    /* Found param */
                    if (QString(type->param_names[j]) == param) {
                        /* Set pd.param and finish the job */
                        pd.param = j;
                        frontend_getparam(conn, &pd);
                        emit(gotParamValue(node, param, pd.value));
                    }
                }
            }
        }
        frontend_free_node_data(nd, num_nodes);
    }
}

void ControlWorker::setParamValue(QString node, QString param, float value)
{
    if (conn) {
        frontend_node_data * nd = NULL;
        size_t num_nodes = 0;
        frontend_getnodes(conn, &nd, &num_nodes);

        for (size_t i = 0; i < num_nodes; i++) {
            if (QString(nd[i].name) == node) {
                frontend_param_data pd;
                pd.block = nd[i].block;

                const struct graph_node_type * type = blockDefs[nd[i].type];
                for (ssize_t j = 0; j < type->n_params; j++) {
                    if (QString(type->param_names[j]) == param) {
                        pd.param = j;
                        pd.value = value;
                        frontend_setparam(conn, &pd);
                        emit(paramSet(node, param));
                    }
                }
            }
        }
        frontend_free_node_data(nd, num_nodes);
    }
}
